import { createTariff } from "@root/api/tariff"
import { CustomTariffUserValuesMap, ServiceKeyToPriceMap } from "@root/model/customTariffs"
import { ServiceKeyToPrivilegesMap } from "@root/model/privilege"
import { produce } from "immer"
import { create } from "zustand"
import { devtools, persist } from "zustand/middleware"
import { Discount, PrivilegeWithAmount, findCartDiscount, findDiscountFactor, findLoyaltyDiscount, findPrivilegeDiscount, findServiceDiscount } from "@frontend/kitui"
import { findNkoDiscount } from "@root/utils/calcCart/calcCart"


interface CustomTariffsStore {
    privilegeByService: ServiceKeyToPrivilegesMap;
    userValuesMap: CustomTariffUserValuesMap;
    summaryPriceBeforeDiscountsMap: ServiceKeyToPriceMap;
    summaryPriceAfterDiscountsMap: ServiceKeyToPriceMap;
}

const initialState: CustomTariffsStore = {
	privilegeByService: {},
	userValuesMap: {},
	summaryPriceBeforeDiscountsMap: {},
	summaryPriceAfterDiscountsMap: {},
}

export const useCustomTariffsStore = create<CustomTariffsStore>()(
	persist(
		devtools(
			(set, get) => initialState,
			{
				name: "Custom tariffs",
				enabled: process.env.NODE_ENV === "development",
			}),
		{
			name: "customTariffs",
			partialize: state => ({
				userValuesMap: state.userValuesMap,
				summaryPriceBeforeDiscountsMap: state.summaryPriceBeforeDiscountsMap,
				summaryPriceAfterDiscountsMap: state.summaryPriceAfterDiscountsMap,
			}),
			migrate: () => initialState,
		}
	)
)

export const setCustomTariffs = (customTariffs: ServiceKeyToPrivilegesMap) => useCustomTariffsStore.setState({ privilegeByService: customTariffs })

export const setCustomTariffsUserValue = (
	serviceKey: string,
	privilegeId: string,
	value: number,
	discounts: Discount[],
	currentCartTotal: number,
	purchasesAmount: number,
	isUserNko: boolean,
) => useCustomTariffsStore.setState(
	produce<CustomTariffsStore>(state => {
		state.userValuesMap[serviceKey] ??= {}
		state.userValuesMap[serviceKey][privilegeId] = value

		let priceWithoutDiscounts = 0
		let priceAfterDiscounts = 0

		const nkoDiscount = findNkoDiscount(discounts)

		if (isUserNko && nkoDiscount) {
			state.privilegeByService[serviceKey].forEach(privilege => {
				const amount = state.userValuesMap[serviceKey]?.[privilege._id] ?? 0
				priceWithoutDiscounts += privilege.price * amount
			})

			priceAfterDiscounts = priceWithoutDiscounts * findDiscountFactor(nkoDiscount)
		} else {
			state.privilegeByService[serviceKey].forEach(privilege => {
				const amount = state.userValuesMap[serviceKey]?.[privilege._id] ?? 0
				priceWithoutDiscounts += privilege.price * amount

				const discount = findPrivilegeDiscount(privilege.privilegeId, privilege.price * amount, discounts)
				priceAfterDiscounts += privilege.price * amount * findDiscountFactor(discount)
			})

			const serviceDiscount = findServiceDiscount(serviceKey, priceAfterDiscounts, discounts)
			priceAfterDiscounts *= findDiscountFactor(serviceDiscount)

			const cartDiscount = findCartDiscount(currentCartTotal, discounts)
			priceAfterDiscounts *= findDiscountFactor(cartDiscount)

			const loyaltyDiscount = findLoyaltyDiscount(purchasesAmount, discounts)
			priceAfterDiscounts *= findDiscountFactor(loyaltyDiscount)
		}

		state.summaryPriceBeforeDiscountsMap[serviceKey] = priceWithoutDiscounts
		state.summaryPriceAfterDiscountsMap[serviceKey] = priceAfterDiscounts
	})
)

export const createAndSendTariff = (serviceKey: string) => {
	const state = useCustomTariffsStore.getState()

	const privileges: PrivilegeWithAmount[] = []

	Object.entries(state.userValuesMap[serviceKey]).forEach(([privilegeId, userValue]) => {
		if (userValue === 0) return

		const privilegeWithoutAmount = state.privilegeByService[serviceKey].find(p => p._id === privilegeId)
		if (!privilegeWithoutAmount) throw new Error(`Privilege not found: ${privilegeId}`)

		const privilege: PrivilegeWithAmount = {
			...privilegeWithoutAmount,
			privilegeId: privilegeWithoutAmount._id,
			amount: userValue,
		}

		privileges.push(privilege)
	})

	const name = [...privileges.map(p => p.name), new Date().toISOString()].join(", ")

	const tariff = {
		name,
		isCustom: true,
		privileges,
	}

	return createTariff(tariff)
}

export const clearCustomTariffs = () => useCustomTariffsStore.setState({ ...initialState })
